@@ -15,6 +15,7 @@
 #include <gtk/gtk.h>
 #include <unistd.h>
 #include <gudev/gudev.h>
+#include <dbus/dbus-glib.h>
 
 #include <sane/sane.h> // For SANE_STATUS_CANCELLED
 
@@ -25,6 +26,8 @@
 
 static const char *default_device = NULL;
 
+static GUdevClient *udev_client;
+
 static SimpleScan *ui;
 
 static Scanner *scanner;
@@ -76,16 +79,16 @@ append_page ()
     /* Copy info from previous page */
     if (page) {
         orientation = page_get_orientation (page);
-        width = page_get_scan_width (page);
-        height = page_get_scan_height (page);
+        width = page_get_width (page);
+        height = page_get_height (page);
         dpi = page_get_dpi (page);
-        
+
         do_crop = page_has_crop (page);
         if (do_crop) {
             named_crop = page_get_named_crop (page);
             page_get_crop (page, &cx, &cy, &cw, &ch);
         }
-    }  
+    }
 
     page = book_append_page (book, width, height, dpi, orientation);
     if (do_crop) {
@@ -111,6 +114,64 @@ scanner_new_page_cb (Scanner *scanner)
 }
 
 
+static gchar *
+get_profile_for_device (const gchar *current_device)
+{
+    gboolean ret;
+    DBusGConnection *connection;
+    DBusGProxy *proxy;
+    GError *error = NULL;
+    GType custom_g_type_string_string;
+    GPtrArray *profile_data_array = NULL;
+    gchar *device_id = NULL;
+    gchar *icc_profile = NULL;
+
+    /* Connect to the color manager on the session bus */
+    connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+    proxy = dbus_g_proxy_new_for_name (connection,
+                                       "org.gnome.ColorManager",
+                                       "/org/gnome/ColorManager",
+                                       "org.gnome.ColorManager");
+
+    /* Get color profile */
+    device_id = g_strdup_printf ("sane:%s", current_device);
+    custom_g_type_string_string = dbus_g_type_get_collection ("GPtrArray",
+                                                              dbus_g_type_get_struct("GValueArray",
+                                                                                     G_TYPE_STRING,
+                                                                                     G_TYPE_STRING,
+                                                                                     G_TYPE_INVALID));
+    ret = dbus_g_proxy_call (proxy, "GetProfilesForDevice", &error,
+                             G_TYPE_STRING, device_id,
+                             G_TYPE_STRING, "",
+                             G_TYPE_INVALID,
+                             custom_g_type_string_string, &profile_data_array,
+                             G_TYPE_INVALID);
+    g_object_unref (proxy);
+    g_free (device_id);
+    if (!ret) {
+        g_debug ("The request failed: %s", error->message);
+        g_error_free (error);
+        return NULL;
+    }
+
+    if (profile_data_array->len > 0) {
+        GValueArray *gva;
+        GValue *gv = NULL;
+
+        /* Just use the preferred profile filename */
+        gva = (GValueArray *) g_ptr_array_index (profile_data_array, 0);
+        gv = g_value_array_get_nth (gva, 1);
+        icc_profile = g_value_dup_string (gv);
+        g_value_unset (gv);
+    }
+    else
+        g_debug ("There are no ICC profiles for the device sane:%s", current_device);
+    g_ptr_array_free (profile_data_array, TRUE);
+
+    return icc_profile;
+}
+
+
 static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
@@ -122,6 +183,11 @@ scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
     /* Add a new page */
     page = append_page ();
     page_set_scan_area (page, info->width, info->height, info->dpi);
+
+    /* Get ICC color profile */
+    /* FIXME: The ICC profile could change */
+    /* FIXME: Don't do a D-bus call for each page, cache color profiles */
+    page_set_color_profile (page, get_profile_for_device (info->device));
 }
 
 
@@ -220,23 +286,25 @@ cancel_cb (SimpleScan *ui)
 
 
 static gboolean
-save_book (const gchar *uri, GError **error)
+save_book_by_extension (GFile *file, GError **error)
 {
     gboolean result;
-    gchar *uri_lower;
+    gchar *uri, *uri_lower;
 
+    uri = g_file_get_uri (file);
     uri_lower = g_utf8_strdown (uri, -1);
     if (g_str_has_suffix (uri_lower, ".pdf"))
-        result = book_save_pdf (book, uri, error);
+        result = book_save (book, "pdf", file, error);
     else if (g_str_has_suffix (uri_lower, ".ps"))
-        result = book_save_ps (book, uri, error);
+        result = book_save (book, "ps", file, error);
     else if (g_str_has_suffix (uri_lower, ".png"))
-        result = book_save_png (book, uri, error);
+        result = book_save (book, "png", file, error);
     else if (g_str_has_suffix (uri_lower, ".tif") || g_str_has_suffix (uri_lower, ".tiff"))
-        result = book_save_tiff (book, uri, error);
+        result = book_save (book, "tiff", file, error);
     else
-        result = book_save_jpeg (book, uri, error);
+        result = book_save (book, "jpeg", file, error);
 
+    g_free (uri);
     g_free (uri_lower);
 
     return result;
@@ -247,18 +315,21 @@ static void
 save_cb (SimpleScan *ui, const gchar *uri)
 {
     GError *error = NULL;
+    GFile *file;
 
     g_debug ("Saving to '%s'", uri);
 
-    if (!save_book (uri, &error)) {
+    file = g_file_new_for_uri (uri);
+    if (!save_book_by_extension (file, &error)) {
         g_warning ("Error saving file: %s", error->message);
         ui_show_error (ui,
                        /* Title of error dialog when save failed */
                        _("Failed to save file"),
                        error->message,
-		       FALSE);
+               FALSE);
         g_error_free (error);
     }
+    g_object_unref (file);
 }
 
 
@@ -297,23 +368,26 @@ email_cb (SimpleScan *ui, const gchar *profile)
 
     /* Save text files as PDFs */
     if (strcmp (profile, "text") == 0) {
-        gchar *path, *uri;
+        gchar *path;
 
         /* Open a temporary file */
         path = get_temporary_filename ("scanned-document", "pdf");
         if (path) {
-            uri = g_filename_to_uri (path, NULL, NULL);
-            saved = book_save_pdf (book, uri, &error);
+            GFile *file;
+
+            file = g_file_new_for_path (path);
+            saved = book_save (book, "pdf", file, &error);
             g_string_append_printf (command_line, " --attach %s", path);
             g_free (path);
-            g_free (uri);
+            g_object_unref (file);
         }
     }
     else {
         gint i;
 
         for (i = 0; i < book_get_n_pages (book); i++) {
-            gchar *path, *uri;
+            gchar *path;
+            GFile *file;
 
             path = get_temporary_filename ("scanned-document", "jpg");
             if (!path) {
@@ -321,11 +395,11 @@ email_cb (SimpleScan *ui, const gchar *profile)
                 break;
             }
 
-            uri = g_filename_to_uri (path, NULL, NULL);
-            saved = page_save_jpeg (book_get_page (book, i), uri, &error);
+            file = g_file_new_for_path (path);
+            saved = page_save (book_get_page (book, i), "jpeg", file, &error);
             g_string_append_printf (command_line, " --attach %s", path);
             g_free (path);
-            g_free (uri);
+            g_object_unref (file);
           
             if (!saved)
                 break;
@@ -353,6 +427,9 @@ email_cb (SimpleScan *ui, const gchar *profile)
 static void
 quit_cb (SimpleScan *ui)
 {
+    g_object_unref (book);
+    g_object_unref (ui);
+    g_object_unref (udev_client);
     scanner_free (scanner);
     gtk_main_quit ();
 }
@@ -380,7 +457,7 @@ usage(int show_gtk)
     fprintf(stderr,
             /* Description on how to use simple-scan displayed on command-line */    
             _("Help Options:\n"
-              "  -d, --debug                     Print debugging messages\n"	      
+              "  -d, --debug                     Print debugging messages\n"
               "  -v, --version                   Show release version\n"
               "  -h, --help                      Show help options\n"
               "  --help-all                      Show all help options\n"
@@ -493,10 +570,10 @@ on_uevent (GUdevClient *client, const gchar *action, GUdevDevice *device)
     scanner_redetect (scanner);
 }
 
+
 int
 main (int argc, char **argv)
 {
-    GUdevClient *udev_client;
     const char *udev_subsystems[] = { "usb", NULL };
     gchar *path;
 
