@@ -39,16 +39,17 @@ static guint signals[LAST_SIGNAL] = { 0, };
 struct SimpleScanPrivate
 {
     GConfClient *client;
-    
+
     GtkBuilder *builder;
 
     GtkWidget *window;
-    GtkWidget *mode_combo;
-    GtkTreeModel *mode_model;
     GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
     GtkWidget *stop_menuitem, *stop_toolbutton;
 
+    GtkWidget *text_toolbar_menuitem, *text_menu_menuitem;
+    GtkWidget *photo_toolbar_menuitem, *photo_menu_menuitem;
+
     GtkWidget *authorize_dialog;
     GtkWidget *authorize_label;
     GtkWidget *username_entry, *password_entry;
@@ -66,6 +67,8 @@ struct SimpleScanPrivate
   
     gboolean have_device_list;
 
+    gchar *document_hint;
+
     gchar *default_file_name;
     gboolean scanning;
 
@@ -266,9 +269,10 @@ static void
 add_default_page (SimpleScan *ui)
 {
     Page *page;
-  
+
     page = book_append_page (ui->priv->book,
-                             ui->priv->default_page_width, ui->priv->default_page_height,
+                             ui->priv->default_page_width,
+                             ui->priv->default_page_height,
                              ui->priv->default_page_dpi,
                              ui->priv->default_page_orientation);
     book_view_select_page (ui->priv->book_view, page);
@@ -288,23 +292,37 @@ new_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 static void
 set_document_hint (SimpleScan *ui, const gchar *document_hint)
 {
-    GtkTreeIter iter;
+    g_free (ui->priv->document_hint);
+    ui->priv->document_hint = g_strdup (document_hint);
 
-    if (gtk_tree_model_get_iter_first (ui->priv->mode_model, &iter)) {
-        do {
-            gchar *d;
-            gboolean have_match;
+    if (strcmp (document_hint, "text") == 0) {
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->text_toolbar_menuitem), TRUE);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->text_menu_menuitem), TRUE);
+    }
+    else if (strcmp (document_hint, "photo") == 0) {
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->photo_toolbar_menuitem), TRUE);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (ui->priv->photo_menu_menuitem), TRUE);
+    }
+}
 
-            gtk_tree_model_get (ui->priv->mode_model, &iter, 0, &d, -1);
-            have_match = strcmp (d, document_hint) == 0;
-            g_free (d);
 
-            if (have_match) {
-                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->mode_combo), &iter);                
-                return;
-            }
-        } while (gtk_tree_model_iter_next (ui->priv->mode_model, &iter));
-     }
+void text_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+text_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_document_hint (ui, "text");
+}
+
+
+void photo_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+photo_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_document_hint (ui, "photo");
 }
 
 
@@ -407,19 +425,6 @@ get_paper_size (SimpleScan *ui, gint *width, gint *height)
 }
 
 
-static gchar *
-get_document_hint (SimpleScan *ui)
-{
-    GtkTreeIter iter;
-    gchar *mode = NULL;
-
-    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->mode_combo), &iter))
-        gtk_tree_model_get (ui->priv->mode_model, &iter, 0, &mode, -1);
-    
-    return mode;
-}
-
-
 static ScanOptions *
 get_scan_options (SimpleScan *ui)
 {
@@ -433,13 +438,11 @@ get_scan_options (SimpleScan *ui)
         { NULL,    SCAN_MODE_COLOR   }
     };
     gint i;
-    gchar *profile_name;
     ScanOptions *options;
 
     /* Find this profile */
     // FIXME: Move this into scan-profile.c
-    profile_name = get_document_hint (ui);
-    for (i = 0; profiles[i].name && strcmp (profiles[i].name, profile_name) != 0; i++);
+    for (i = 0; profiles[i].name && strcmp (profiles[i].name, ui->priv->document_hint) != 0; i++);
   
     options = g_malloc0 (sizeof (ScanOptions));
     options->scan_mode = profiles[i].mode;
@@ -450,8 +453,6 @@ get_scan_options (SimpleScan *ui)
         options->dpi = get_text_dpi (ui);
     get_paper_size (ui, &options->paper_width, &options->paper_height);
   
-    g_free (profile_name);
-
     return options;
 }
 
@@ -610,19 +611,26 @@ get_temporary_filename (const gchar *prefix, const gchar *extension)
 static void
 show_page_cb (BookView *view, Page *page, SimpleScan *ui)
 {
-    gchar *path, *uri;
+    gchar *path;
+    GFile *file;
     GdkScreen *screen;
     GError *error = NULL;
   
     path = get_temporary_filename ("scanned-page", "tiff");
     if (!path)
         return;
+    file = g_file_new_for_path (path);
+    g_free (path);
 
     screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
 
-    uri = g_filename_to_uri (path, NULL, NULL);
-    if (page_save_tiff (page, uri, &error))
+    if (page_save (page, "tiff", file, &error)) {
+        gchar *uri = g_file_get_uri (file);
         gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+        g_free (uri);
+    }
+
+    g_object_unref (file);
 
     if (error) {
         ui_show_error (ui,
@@ -632,9 +640,6 @@ show_page_cb (BookView *view, Page *page, SimpleScan *ui)
                        FALSE);
         g_clear_error (&error);
     }
-
-    g_free (uri);
-    g_free (path);
 }
 
 
@@ -1013,10 +1018,7 @@ G_MODULE_EXPORT
 void
 email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    gchar *mode;
-    mode = get_document_hint (ui);
-    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, mode);
-    g_free (mode);
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, ui->priv->document_hint);
 }
 
 
@@ -1109,7 +1111,7 @@ about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 static void
 quit (SimpleScan *ui)
 {
-    char *device, *document_type;
+    char *device;
     gint paper_width = 0, paper_height = 0;
     gint i;
 
@@ -1121,9 +1123,7 @@ quit (SimpleScan *ui)
         g_free (device);
     }
 
-    document_type = get_document_hint (ui);
-    gconf_client_set_string(ui->priv->client, GCONF_DIR "/document_type", document_type, NULL);
-    g_free (document_type);
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/document_type", ui->priv->document_hint, NULL);
     gconf_client_set_int (ui->priv->client, GCONF_DIR "/text_dpi", get_text_dpi (ui), NULL);
     gconf_client_set_int (ui->priv->client, GCONF_DIR "/photo_dpi", get_photo_dpi (ui), NULL);
     gconf_client_set_string (ui->priv->client, GCONF_DIR "/page_side", get_page_side (ui), NULL);
@@ -1193,8 +1193,8 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 static void
 page_size_changed_cb (Page *page, SimpleScan *ui)
 {
-    ui->priv->default_page_width = page_get_scan_width (page);
-    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_width = page_get_width (page);
+    ui->priv->default_page_height = page_get_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
 }
 
@@ -1209,8 +1209,8 @@ page_orientation_changed_cb (Page *page, SimpleScan *ui)
 static void
 page_added_cb (Book *book, Page *page, SimpleScan *ui)
 {
-    ui->priv->default_page_width = page_get_scan_width (page);
-    ui->priv->default_page_height = page_get_scan_height (page);
+    ui->priv->default_page_width = page_get_width (page);
+    ui->priv->default_page_height = page_get_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
     ui->priv->default_page_orientation = page_get_orientation (page);
     g_signal_connect (page, "size-changed", G_CALLBACK (page_size_changed_cb), ui);
@@ -1309,8 +1309,6 @@ ui_load (SimpleScan *ui)
     gtk_builder_connect_signals (builder, ui);
 
     ui->priv->window = GTK_WIDGET (gtk_builder_get_object (builder, "simple_scan_window"));
-    ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
-    ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
     ui->priv->preview_box = GTK_WIDGET (gtk_builder_get_object (builder, "preview_vbox"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
     ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
@@ -1319,10 +1317,10 @@ ui_load (SimpleScan *ui)
     ui->priv->stop_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "stop_scan_menuitem"));
     ui->priv->stop_toolbutton = GTK_WIDGET (gtk_builder_get_object (builder, "stop_toolbutton"));
 
-    renderer = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
-    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, "text", 1);
-    gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->mode_combo), 0);
+    ui->priv->text_toolbar_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "text_toolbutton_menuitem"));
+    ui->priv->text_menu_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "text_menuitem"));
+    ui->priv->photo_toolbar_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "photo_toolbutton_menuitem"));
+    ui->priv->photo_menu_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "photo_menuitem"));
 
     ui->priv->authorize_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_dialog"));
     ui->priv->authorize_label = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_label"));
@@ -1417,6 +1415,7 @@ ui_load (SimpleScan *ui)
         for (i = 0; orientation_keys[i].key != NULL && strcmp (orientation_keys[i].key, scan_direction) != 0; i++);
         if (orientation_keys[i].key != NULL)
             ui->priv->default_page_orientation = orientation_keys[i].orientation;
+        g_free (scan_direction);
     }
     ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_width", NULL);
     if (ui->priv->default_page_width <= 0)
@@ -1459,7 +1458,7 @@ ui_new ()
 Book *
 ui_get_book (SimpleScan *ui)
 {
-    return ui->priv->book;
+    return g_object_ref (ui->priv->book);
 }
 
 
@@ -1560,9 +1559,31 @@ g_cclosure_user_marshal_VOID__STRING_POINTER (GClosure     *closure,
 }
 
 
+static void
+ui_finalize (GObject *object)
+{
+    SimpleScan *ui = SIMPLE_SCAN (object);
+
+    g_object_unref (ui->priv->client);
+    ui->priv->client = NULL;
+    g_object_unref (ui->priv->builder);
+    ui->priv->builder = NULL;
+    g_object_unref (ui->priv->book);
+    ui->priv->book = NULL;
+    g_object_unref (ui->priv->book_view);
+    ui->priv->book_view = NULL;    
+  
+    G_OBJECT_CLASS (ui_parent_class)->finalize (object);
+}
+
+
 static void
 ui_class_init (SimpleScanClass *klass)
 {
+    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+    object_class->finalize = ui_finalize;
+
     signals[START_SCAN] =
         g_signal_new ("start-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -1620,6 +1641,7 @@ ui_init (SimpleScan *ui)
     ui->priv->client = gconf_client_get_default();
     gconf_client_add_dir(ui->priv->client, GCONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
+    ui->priv->document_hint = g_strdup ("photo");
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
     ui_load (ui);   
